from functools import wraps
import services
import sims4.resources
from traits.trait_tracker import TraitPickerSuperInteraction
from sims4.tuning.tunable import Tunable, TunableList, TunablePackSafeReference
from sims4.tuning.instance_manager import InstanceManager
from sims4.resources import Types

def inject(target_function, new_function):
    @wraps(target_function)
    def _inject(*args, **kwargs):
        return new_function(target_function, *args, **kwargs)
    return _inject

def inject_to(target_object, target_function_name):
    def _inject_to(new_function):
        target_function = getattr(target_object, target_function_name)
        setattr(target_object, target_function_name, inject(target_function, new_function))
        return new_function
    return _inject_to

class CustomTraitPickerSuperInteraction(TraitPickerSuperInteraction):
    INSTANCE_TUNABLES = {
        'traits': TunableList(tunable=TunablePackSafeReference(manager=services.trait_manager()), allow_none=True),
        'only_one_allowed': Tunable(tunable_type=bool, default=False)
    }
    @classmethod
    def _trait_selection_gen(cls, target):
        trait_tracker = target.sim_info.trait_tracker
        if cls.is_add:
            for trait in cls.traits:
                if trait_tracker.can_add_trait(trait):
                    yield trait
        else:
            for trait in trait_tracker.equipped_traits:
                yield trait
    def on_choice_selected(self, choice_tag, **kwargs):
        if self.only_one_allowed:
            for trait in self.traits:
                self.target.sim_info.remove_trait(trait)
        super().on_choice_selected(choice_tag, **kwargs)

SeSeSimmi_BRAT_ObjectIds_1 = (14965,)
SeSeSimmi_BRAT_InteractionIds_1 = (3625690734,)
@inject_to(InstanceManager, 'load_data_into_class_instances')
def SeSeSimmi_BRAT_AddSuperAffordances_1(original, self):
    original(self)
    if self.TYPE == Types.OBJECT:
        affordance_manager = services.affordance_manager()
        sa_list = []
        for sa_id in SeSeSimmi_BRAT_InteractionIds_1:
            key = sims4.resources.get_resource_key(sa_id, Types.INTERACTION)
            sa_tuning = affordance_manager.get(key)
            if not sa_tuning is None:
                sa_list.append(sa_tuning)
        sa_tuple = tuple(sa_list)
        for obj_id in SeSeSimmi_BRAT_ObjectIds_1:
            key = sims4.resources.get_resource_key(obj_id, Types.OBJECT)
            obj_tuning = self._tuned_classes.get(key)
            if not obj_tuning is None:
                obj_tuning._super_affordances = obj_tuning._super_affordances + sa_tuple

SeSeSimmi_BRAT_163702_SnippetId = 163702
SeSeSimmi_BRAT_163702_MixerId = (2813280442,3675229758,2899683228,4041737115,3557570896,4166419149,2369334594,2855088578,3990682580,3453002384,2200140347,2957200428,2824876318,4096907992,2818067067,3382584211,2300769723,3771277977,3754324933,4199198502,4245008087,2817550116,2340485121,2556845718,3181736895,2533420632,3239944912,3620338607,3918804909,3781724071,2703102176,2310058186,3744942986,2555954328,2588435985,3751399818,2334781690,2685094111,)
@inject_to(InstanceManager, 'load_data_into_class_instances')
def SeSeSimmi_BRAT_AddMixer_163702(original, self):
    original(self)
    if self.TYPE == Types.SNIPPET:
        key = sims4.resources.get_resource_key(SeSeSimmi_BRAT_163702_SnippetId, Types.SNIPPET)
        snippet_tuning = self._tuned_classes.get(key)
        if snippet_tuning is None:
            return
        for m_id in SeSeSimmi_BRAT_163702_MixerId:
            affordance_manager = services.affordance_manager()
            key = sims4.resources.get_resource_key(m_id, Types.INTERACTION)
            mixer_tuning = affordance_manager.get(key)
            if mixer_tuning is None:
                return
            if mixer_tuning in snippet_tuning.value:
                return
            snippet_tuning.value = snippet_tuning.value + (mixer_tuning, )
SeSeSimmi_BRAT_163713_SnippetId = 163713
SeSeSimmi_BRAT_163713_MixerId = (3094525828,3039489480,)
@inject_to(InstanceManager, 'load_data_into_class_instances')
def SeSeSimmi_BRAT_AddMixer_163713(original, self):
    original(self)
    if self.TYPE == Types.SNIPPET:
        key = sims4.resources.get_resource_key(SeSeSimmi_BRAT_163713_SnippetId, Types.SNIPPET)
        snippet_tuning = self._tuned_classes.get(key)
        if snippet_tuning is None:
            return
        for m_id in SeSeSimmi_BRAT_163713_MixerId:
            affordance_manager = services.affordance_manager()
            key = sims4.resources.get_resource_key(m_id, Types.INTERACTION)
            mixer_tuning = affordance_manager.get(key)
            if mixer_tuning is None:
                return
            if mixer_tuning in snippet_tuning.value:
                return
            snippet_tuning.value = snippet_tuning.value + (mixer_tuning, )
SeSeSimmi_BRAT_163704_SnippetId = 163704
SeSeSimmi_BRAT_163704_MixerId = (3274180959,3520599669,)
@inject_to(InstanceManager, 'load_data_into_class_instances')
def SeSeSimmi_BRAT_AddMixer_163704(original, self):
    original(self)
    if self.TYPE == Types.SNIPPET:
        key = sims4.resources.get_resource_key(SeSeSimmi_BRAT_163704_SnippetId, Types.SNIPPET)
        snippet_tuning = self._tuned_classes.get(key)
        if snippet_tuning is None:
            return
        for m_id in SeSeSimmi_BRAT_163704_MixerId:
            affordance_manager = services.affordance_manager()
            key = sims4.resources.get_resource_key(m_id, Types.INTERACTION)
            mixer_tuning = affordance_manager.get(key)
            if mixer_tuning is None:
                return
            if mixer_tuning in snippet_tuning.value:
                return
            snippet_tuning.value = snippet_tuning.value + (mixer_tuning, )

